#!/bin/bash
#

# need an in source build

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <s2:e5|s3:e5|s4:e5|s4:e6> <debug|opt|prof> [tar]"
   echo "        s4 -> art v1_11_02 and python v2_7_8"
   echo "        s3 -> art v1_10_00b and python v2_7_6"
   echo "        s2 -> art v1_09_04 and python v2_7_6"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
      (( darwin = 1 ))
      flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=ifdhc
origpkgver=v1_4_3
pkgver=${origpkgver}b
ssibuildshims_version=v0_07_04
pkgtarver=`echo ${origpkgver} | sed -e 's/_/./g'`
pkgtarfile=${package}-${origpkgver}.tar 

get_this_dir

get_ssibuildshims

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

cd ${pkgdir} 
tar xf ${tardir}/${pkgtarfile} || die "ERROR: failed to unwind ${pkgtarfile}"
patch="${patchdir}/ifdhc.patch"
patch -b -p0 < "${patch}" || die "ERROR: application of patch ${patch} failed."

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS} || die "ERROR: fake setup failed"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

case ${basequal} in
s2:e5) stdcxx="-std=c++11" ;;
s3:e5) stdcxx="-std=c++11" ;;
s4:e5) stdcxx="-std=c++11" ;;
s4:e6) stdcxx="-std=c++1y" ;;
*) 
   echo "ERROR: unrecognized qualifier ${basequal}"
   usage
   exit 1
esac

case ${extraqual} in
debug) ARCH="${stdcxx} -O0 -g";;
prof)  ARCH="${stdcxx} -g -p -pg";;
opt)   ARCH="${stdcxx} -O2";;
esac
export ARCH

export PYTHON_CONF=`echo $PYTHON_LIB/python*/config`

swig_minor=`swig -version | grep Version | cut -f 3 -d " " | cut -f 2 -d "."`
swig_patch=`swig -version | grep Version | cut -f 3 -d " " | cut -f 3 -d "."`

if [ ${swig_minor} -eq 3 ] && [ ${swig_patch} -ge 40 ]
then
 mv ${pkgdir}/ifdh/ifdh_wrap.patch ${pkgdir}/ifdh/ifdh_wrap.patch.orig
 cp -p ${patchdir}/ifdh_wrap.patch.malloc ${pkgdir}/ifdh/ifdh_wrap.patch
elif [ ${swig_minor} -gt 3 ]
then
 mv ${pkgdir}/ifdh/ifdh_wrap.patch ${pkgdir}/ifdh/ifdh_wrap.patch.orig
 cp -p ${patchdir}/ifdh_wrap.patch.malloc ${pkgdir}/ifdh/ifdh_wrap.patch
fi

set -x

cd ${pkgdir} 

make IFDHC_VERSION=${origpkgver} all || die "ERROR: make all failed"
make install || die "ERROR: make install failed"
make clean || die "ERROR: make clean failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi


# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
echo "unsetup"
unsetup ${package}
echo "setup"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || die "ERROR: setup failed"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
